

/*
  Basic ESP8266 MQTT example

  This sketch demonstrates the capabilities of the pubsub library in combination
  with the ESP8266 board/library.

  It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off

  It will reconnect to the server if the connection is lost using a blocking
  reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
  achieve the same result without blocking the main loop.

  To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"

*/
#include <ESP8266WiFi.h>
#include <PubSubClient.h>t
#include <Wire.h>
#include <Adafruit_BMP085.h>

// Update these with values suitable for your network.
Adafruit_BMP085 bmp;

const String ssids[] = { "MD_WIFI" };
const String passwords[] = { "MPReoa43"};


const char* ssid = "CENSIS";
const char* password = "Censis2016";
const char* mqtt_server = "192.168.1.150";
const char* host = "api.silvrtree.co.uk";

WiFiClient espClient;
WiFiClient webClient;
const int httpPort = 80;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;
String returnString = "";
int blueLED = 2;

void InitializeBMP085() {
  if (!bmp.begin()) {
    Serial.println("Could not find a valid BMP085 sensor, check wiring!");
    while (1) {}
  }
}

void setup() {
  //Wire.pins(sda, scl);
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output
   pinMode(blueLED, OUTPUT);
  digitalWrite(blueLED, HIGH);
  digitalWrite(BUILTIN_LED, HIGH);

  Serial.begin(115200);
  InitializeBMP085();
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void setup_wifi() {
  int concount = 0;
  int SSIDmode = 0;
  char ssid_b[20];
  char password_b[20];
  while (WiFi.status() != WL_CONNECTED) {


    delay(10);
    // We start by connecting to a WiFi network

    ssids[SSIDmode].toCharArray(ssid_b, 20);
    passwords[SSIDmode].toCharArray(password_b, 20);

    Serial.println();
    Serial.print("Connecting to ");
    Serial.println(ssid_b);


    WiFi.begin(ssid_b, password_b);
    concount = 0;
    while (WiFi.status() != WL_CONNECTED && concount < 40) {
      digitalWrite(blueLED, LOW);
      delay(50);
      digitalWrite(blueLED, HIGH);
      delay(450);
      Serial.print(".");

    concount++;
    }
    

    SSIDmode++;

    if (SSIDmode > 4) {
      SSIDmode = 0;
    }



  }

  /*
    char ssid_b[20];
    char password_b[20];
    delay(10);
    // We start by connecting to a WiFi network

    ssids[0].toCharArray(ssid_b, 20);
    passwords[0].toCharArray(password_b, 20);

    Serial.println();
    Serial.print("Connecting to ");
    Serial.println(ssid_b);


    WiFi.begin(ssid_b, password_b);

    while (WiFi.status() != WL_CONNECTED) {


    delay(500);
    Serial.print(".");


    }*/

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  digitalWrite(BUILTIN_LED, LOW);
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();

  // Switch on the LED if an 1 was received as first character
  if ((char)payload[0] == '1') {
    digitalWrite(BUILTIN_LED, LOW);   // Turn the LED on (Note that LOW is the voltage level
    // but actually the LED is on; this is because
    // it is acive low on the ESP-01)
  } else {
    digitalWrite(BUILTIN_LED, HIGH);  // Turn the LED off by making the voltage HIGH
  }

}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Client")) {
    //  digitalWrite(blueLED, LOW);
      Serial.println("connected");
      // Once connected, publish an announcement...
      // client.publish("livingroomLogin", "hello");
      // ... and resubscribe
      client.subscribe("bedroomTemp");
    } else {
  //    digitalWrite(blueLED, HIGH);
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  long now = millis();

  if (now - lastMsg > 10000) {
     digitalWrite(BUILTIN_LED, LOW);
      delay(15);
      digitalWrite(BUILTIN_LED, HIGH);
      delay(50);
      
    float tempReading = bmp.readTemperature();
    float pressureReading = bmp.readPressure() / 100.0;
    char szEventInfo[64];

    String strTemp = "{\"temp\":" + String(tempReading, 2) + "}";

    strTemp.toCharArray(szEventInfo, 64);

    //snprintf (msg, 75, "{temp:%.2f}\n", tempReading);
    Serial.print("Publish message: ");
    Serial.println(szEventInfo);
    client.publish("bedroomTemp", szEventInfo);

    lastMsg = now;
  }

}


